@@ -1,13 +1,19 @@
-#define QT_FEATURE_android_style_assets -1
-
-#define QT_FEATURE_gc_binaries -1
-
 #define QT_FEATURE_use_bfd_linker -1
 
 #define QT_FEATURE_use_gold_linker -1
 
 #define QT_FEATURE_use_lld_linker -1
 
+#define QT_FEATURE_use_mold_linker -1
+
+#define QT_FEATURE_android_style_assets -1
+
+#define QT_FEATURE_gc_binaries -1
+
+#define QT_FEATURE_developer_build -1
+
+#define QT_FEATURE_no_prefix -1
+
 #define QT_FEATURE_private_tests -1
 
 #define QT_FEATURE_debug -1
@@ -82,6 +88,8 @@
 
 #define QT_FEATURE_zstd 1
 
+#define QT_FEATURE_stdlib_libcpp -1
+
 #define QT_FEATURE_dbus 1
 
 #define QT_FEATURE_dbus_linked 1
